'use strict';

exports.__esModule = true;
exports.decodeContent = decodeContent;
exports.encodeContent = encodeContent;

var _zlib = require('zlib');

var _zlib2 = _interopRequireDefault(_zlib);

var _promisifiedFunctions = require('../../utils/promisified-functions');

var _iconvLite = require('iconv-lite');

var _iconvLite2 = _interopRequireDefault(_iconvLite);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var __awaiter = undefined && undefined.__awaiter || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) {
            try {
                step(generator.next(value));
            } catch (e) {
                reject(e);
            }
        }
        function rejected(value) {
            try {
                step(generator["throw"](value));
            } catch (e) {
                reject(e);
            }
        }
        function step(result) {
            result.done ? resolve(result.value) : new P(function (resolve) {
                resolve(result.value);
            }).then(fulfilled, rejected);
        }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};

const GZIP_CONTENT_ENCODING = 'gzip';
const DEFLATE_CONTENT_ENCODING = 'deflate';
const BROTLI_CONTENT_ENCODING = 'br';
// NOTE: IIS has a bug when it sends 'raw deflate' compressed data for the 'Deflate' Accept-Encoding header.
// (see: http://zoompf.com/2012/02/lose-the-wait-http-compression)
function inflateWithFallback(data) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            return yield (0, _promisifiedFunctions.inflate)(data);
        } catch (err) {
            if (err.code === 'Z_DATA_ERROR') return yield (0, _promisifiedFunctions.inflateRaw)(data);
            throw err;
        }
    });
}
function decodeContent(content, encoding, charset) {
    return __awaiter(this, void 0, void 0, function* () {
        if (encoding === GZIP_CONTENT_ENCODING) {
            // NOTE: https://github.com/request/request/pull/2492/files
            // Be more lenient with decoding compressed responses, since (very rarely)
            // servers send slightly invalid gzip responses that are still accepted
            // by common browsers.
            // Always using Z_SYNC_FLUSH is what cURL does.
            // GH-1915
            content = yield (0, _promisifiedFunctions.gunzip)(content, { flush: _zlib2.default.Z_SYNC_FLUSH, finishFlush: _zlib2.default.Z_SYNC_FLUSH });
        } else if (encoding === DEFLATE_CONTENT_ENCODING) content = yield inflateWithFallback(content);else if (encoding === BROTLI_CONTENT_ENCODING) content = Buffer.from(require('brotli').decompress(content));
        charset.fromBOM(content);
        return _iconvLite2.default.decode(content, charset.get());
    });
}
function encodeContent(content, encoding, charset) {
    return __awaiter(this, void 0, void 0, function* () {
        content = _iconvLite2.default.encode(content, charset.get(), { addBOM: charset.isFromBOM() });
        if (encoding === GZIP_CONTENT_ENCODING) return (0, _promisifiedFunctions.gzip)(content);
        if (encoding === DEFLATE_CONTENT_ENCODING) return (0, _promisifiedFunctions.deflate)(content);
        if (encoding === BROTLI_CONTENT_ENCODING) return Buffer.from(require('brotli').compress(content));
        return content;
    });
}