'use strict';

exports.__esModule = true;
exports.run = run;

var _context = require('./context');

var _context2 = _interopRequireDefault(_context);

var _resources = require('../processing/resources');

var _messages = require('../messages');

var _connectionResetGuard = require('./connection-reset-guard');

var _connectionResetGuard2 = _interopRequireDefault(_connectionResetGuard);

var _sameOriginCheckFailedStatusCode = require('./xhr/same-origin-check-failed-status-code');

var _sameOriginCheckFailedStatusCode2 = _interopRequireDefault(_sameOriginCheckFailedStatusCode);

var _http = require('../utils/http');

var _websocket = require('./websocket');

var _specialPage = require('./special-page');

var _specialPage2 = _interopRequireDefault(_specialPage);

var _info = require('../session/events/info');

var requestEventInfo = _interopRequireWildcard(_info);

var _names = require('../session/events/names');

var _names2 = _interopRequireDefault(_names);

var _configureResponseEvent = require('../session/events/configure-response-event');

var _configureResponseEvent2 = _interopRequireDefault(_configureResponseEvent);

var _configureResponseEventOptions = require('../session/events/configure-response-event-options');

var _configureResponseEventOptions2 = _interopRequireDefault(_configureResponseEventOptions);

var _utils = require('./utils');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var __awaiter = undefined && undefined.__awaiter || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) {
            try {
                step(generator.next(value));
            } catch (e) {
                reject(e);
            }
        }
        function rejected(value) {
            try {
                step(generator["throw"](value));
            } catch (e) {
                reject(e);
            }
        }
        function step(result) {
            result.done ? resolve(result.value) : new P(function (resolve) {
                resolve(result.value);
            }).then(fulfilled, rejected);
        }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};

const EVENT_SOURCE_REQUEST_TIMEOUT = 60 * 60 * 1000;
const stages = [function handleSocketError(ctx) {
    // NOTE: In some case on MacOS, browser reset connection with server and we need to catch this exception.
    if (!ctx.isWebSocket) return;
    ctx.res.on('error', e => {
        if (e.code === 'ECONNRESET' && !ctx.mock) {
            if (ctx.destRes) ctx.destRes.destroy();else ctx.isWebSocketConnectionReset = true;
        } else throw e;
    });
}, function fetchProxyRequestBody(ctx) {
    return __awaiter(this, void 0, void 0, function* () {
        if (ctx.isPage && !ctx.isIframe && !ctx.isHtmlImport) ctx.session.onPageRequest(ctx);
        ctx.reqBody = yield (0, _http.fetchBody)(ctx.req);
    });
}, function sendDestinationRequest(ctx) {
    return __awaiter(this, void 0, void 0, function* () {
        if (ctx.isSpecialPage) {
            ctx.destRes = (0, _specialPage2.default)();
            return;
        }
        ctx.reqOpts = (0, _utils.createReqOpts)(ctx);
        if (ctx.session.hasRequestEventListeners()) {
            const requestInfo = requestEventInfo.createRequestInfo(ctx, ctx.reqOpts);
            ctx.requestFilterRules = ctx.session.getRequestFilterRules(requestInfo);
            yield ctx.forEachRequestFilterRule(rule => __awaiter(this, void 0, void 0, function* () {
                yield (0, _utils.callOnRequestEventCallback)(ctx, rule, requestInfo);
                ctx.setupMockIfNecessary(rule);
            }));
        }
        if (ctx.mock) ctx.mockResponse();else yield (0, _utils.sendRequest)(ctx);
    });
}, function checkSameOriginPolicyCompliance(ctx) {
    return __awaiter(this, void 0, void 0, function* () {
        ctx.buildContentInfo();
        if (ctx.isPassSameOriginPolicy()) return;
        yield ctx.forEachRequestFilterRule(rule => __awaiter(this, void 0, void 0, function* () {
            const configureResponseEvent = new _configureResponseEvent2.default(ctx, rule, _configureResponseEventOptions2.default.DEFAULT);
            yield ctx.session.callRequestEventCallback(_names2.default.onConfigureResponse, rule, configureResponseEvent);
            yield (0, _utils.callOnResponseEventCallbackForFailedSameOriginCheck)(ctx, rule, configureResponseEvent);
        }));
        ctx.closeWithError(_sameOriginCheckFailedStatusCode2.default);
    });
}, function decideOnProcessingStrategy(ctx) {
    return __awaiter(this, void 0, void 0, function* () {
        ctx.goToNextStage = false;
        if (ctx.isWebSocket) {
            (0, _websocket.respondOnWebSocket)(ctx);
            return;
        }
        if (ctx.contentInfo.requireProcessing && ctx.destRes.statusCode === 204) ctx.destRes.statusCode = 200;
        // NOTE: Just pipe the content body to the browser if we don't need to process it.
        else if (!ctx.contentInfo.requireProcessing) {
                if (!ctx.isSpecialPage) {
                    yield (0, _utils.callOnConfigureResponseEventForNonProcessedRequest)(ctx);
                    ctx.sendResponseHeaders();
                    if (ctx.contentInfo.isNotModified) yield (0, _utils.callOnResponseEventCallbackForMotModifiedResource)(ctx);else {
                        const onResponseEventDataWithBody = ctx.getOnResponseEventData({ includeBody: true });
                        const onResponseEventDataWithoutBody = ctx.getOnResponseEventData({ includeBody: false });
                        if (onResponseEventDataWithBody.length) yield (0, _utils.callOnResponseEventCallbackWithBodyForNonProcessedRequest)(ctx, onResponseEventDataWithBody);else if (onResponseEventDataWithoutBody.length) yield (0, _utils.callOnResponseEventCallbackWithoutBodyForNonProcessedResource)(ctx, onResponseEventDataWithoutBody);else ctx.destRes.pipe(ctx.res);
                    }
                    // NOTE: sets 60 minutes timeout for the "event source" requests instead of 2 minutes by default
                    if (ctx.dest.isEventSource) {
                        ctx.req.setTimeout(EVENT_SOURCE_REQUEST_TIMEOUT);
                        ctx.req.on('close', () => ctx.destRes.destroy());
                    }
                } else {
                    ctx.sendResponseHeaders();
                    ctx.res.end();
                }
                return;
            }
        ctx.goToNextStage = true;
    });
}, function fetchContent(ctx) {
    return __awaiter(this, void 0, void 0, function* () {
        ctx.destResBody = yield (0, _http.fetchBody)(ctx.destRes);
        if (ctx.requestFilterRules.length) ctx.saveNonProcessedDestResBody(ctx.destResBody);
        // NOTE: Sometimes the underlying socket emits an error event. But if we have a response body,
        // we can still process such requests. (B234324)
        if (ctx.hasDestReqErr && ctx.isDestResBodyMalformed()) {
            (0, _utils.error)(ctx, (0, _messages.getText)(_messages.MESSAGE.destConnectionTerminated, ctx.dest.url));
            ctx.goToNextStage = false;
        }
    });
}, function processContent(ctx) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            ctx.destResBody = yield (0, _resources.process)(ctx);
        } catch (err) {
            (0, _utils.error)(ctx, err);
        }
    });
}, function sendProxyResponse(ctx) {
    return __awaiter(this, void 0, void 0, function* () {
        const configureResponseEvents = yield Promise.all(ctx.requestFilterRules.map(rule => __awaiter(this, void 0, void 0, function* () {
            const configureResponseEvent = new _configureResponseEvent2.default(ctx, rule, _configureResponseEventOptions2.default.DEFAULT);
            yield ctx.session.callRequestEventCallback(_names2.default.onConfigureResponse, rule, configureResponseEvent);
            return configureResponseEvent;
        })));
        ctx.sendResponseHeaders();
        (0, _connectionResetGuard2.default)(() => __awaiter(this, void 0, void 0, function* () {
            yield Promise.all(configureResponseEvents.map(configureResponseEvent => __awaiter(this, void 0, void 0, function* () {
                yield (0, _utils.callResponseEventCallbackForProcessedRequest)(ctx, configureResponseEvent);
            })));
            ctx.res.write(ctx.destResBody);
            ctx.res.end();
        }));
    });
}];
function run(req, res, serverInfo, openSessions) {
    return __awaiter(this, void 0, void 0, function* () {
        const ctx = new _context2.default(req, res, serverInfo);
        if (!ctx.dispatch(openSessions)) {
            (0, _http.respond404)(res);
            return;
        }
        for (let i = 0; i < stages.length; i++) {
            yield stages[i](ctx);
            if (!ctx.goToNextStage) return;
        }
    });
}