'use strict';

exports.__esModule = true;
exports.createReqOpts = createReqOpts;
exports.sendRequest = sendRequest;
exports.error = error;
exports.callResponseEventCallbackForProcessedRequest = callResponseEventCallbackForProcessedRequest;
exports.callOnRequestEventCallback = callOnRequestEventCallback;
exports.callOnResponseEventCallbackForFailedSameOriginCheck = callOnResponseEventCallbackForFailedSameOriginCheck;
exports.callOnConfigureResponseEventForNonProcessedRequest = callOnConfigureResponseEventForNonProcessedRequest;
exports.callOnResponseEventCallbackWithBodyForNonProcessedRequest = callOnResponseEventCallbackWithBodyForNonProcessedRequest;
exports.callOnResponseEventCallbackWithoutBodyForNonProcessedResource = callOnResponseEventCallbackWithoutBodyForNonProcessedResource;
exports.callOnResponseEventCallbackForMotModifiedResource = callOnResponseEventCallbackForMotModifiedResource;

var _upload = require('../upload');

var _matchUrlWildcard = require('match-url-wildcard');

var _matchUrlWildcard2 = _interopRequireDefault(_matchUrlWildcard);

var _headerTransforms = require('./header-transforms');

var headerTransforms = _interopRequireWildcard(_headerTransforms);

var _fileRequest = require('./file-request');

var _fileRequest2 = _interopRequireDefault(_fileRequest);

var _destinationRequest = require('./destination-request');

var _destinationRequest2 = _interopRequireDefault(_destinationRequest);

var _info = require('../session/events/info');

var requestEventInfo = _interopRequireWildcard(_info);

var _promisifyStream = require('../utils/promisify-stream');

var _promisifyStream2 = _interopRequireDefault(_promisifyStream);

var _configureResponseEvent = require('../session/events/configure-response-event');

var _configureResponseEvent2 = _interopRequireDefault(_configureResponseEvent);

var _requestEvent = require('../session/events/request-event');

var _requestEvent2 = _interopRequireDefault(_requestEvent);

var _responseEvent = require('../session/events/response-event');

var _responseEvent2 = _interopRequireDefault(_responseEvent);

var _names = require('../session/events/names');

var _names2 = _interopRequireDefault(_names);

var _configureResponseEventOptions = require('../session/events/configure-response-event-options');

var _configureResponseEventOptions2 = _interopRequireDefault(_configureResponseEventOptions);

var _buffer = require('../utils/buffer');

var _stream = require('stream');

var _sameOriginCheckFailedStatusCode = require('./xhr/same-origin-check-failed-status-code');

var _sameOriginCheckFailedStatusCode2 = _interopRequireDefault(_sameOriginCheckFailedStatusCode);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var __awaiter = undefined && undefined.__awaiter || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) {
            try {
                step(generator.next(value));
            } catch (e) {
                reject(e);
            }
        }
        function rejected(value) {
            try {
                step(generator["throw"](value));
            } catch (e) {
                reject(e);
            }
        }
        function step(result) {
            result.done ? resolve(result.value) : new P(function (resolve) {
                resolve(result.value);
            }).then(fulfilled, rejected);
        }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
function createReqOpts(ctx) {
    const bodyWithUploads = (0, _upload.inject)(ctx.req.headers['content-type'], ctx.reqBody);
    // NOTE: First, we should rewrite the request body, because the 'content-length' header will be built based on it.
    if (bodyWithUploads) ctx.reqBody = bodyWithUploads;
    // NOTE: All headers, including 'content-length', are built here.
    const headers = headerTransforms.forRequest(ctx);
    const proxy = ctx.session.externalProxySettings;
    const options = {
        url: ctx.dest.url,
        protocol: ctx.dest.protocol,
        hostname: ctx.dest.hostname,
        host: ctx.dest.host,
        port: ctx.dest.port,
        path: ctx.dest.partAfterHost,
        method: ctx.req.method,
        credentials: ctx.session.getAuthCredentials(),
        body: ctx.reqBody,
        isXhr: ctx.isXhr,
        rawHeaders: ctx.req.rawHeaders,
        proxy: void 0,
        headers
    };
    if (proxy && !(0, _matchUrlWildcard2.default)(ctx.dest.url, proxy.bypassRules)) {
        options.proxy = proxy;
        if (ctx.dest.protocol === 'http:') {
            options.path = options.protocol + '//' + options.host + options.path;
            options.host = proxy.host;
            options.hostname = proxy.hostname;
            options.port = proxy.port;
            if (proxy.authHeader) headers['proxy-authorization'] = proxy.authHeader;
        }
    }
    return options;
}
function sendRequest(ctx) {
    return new Promise(resolve => {
        const req = ctx.isFileProtocol ? new _fileRequest2.default(ctx.reqOpts) : new _destinationRequest2.default(ctx.reqOpts);
        ctx.goToNextStage = false;
        req.on('response', res => {
            if (ctx.isWebSocketConnectionReset) {
                res.destroy();
                resolve();
            }
            ctx.destRes = res;
            ctx.goToNextStage = true;
            resolve();
        });
        req.on('error', () => {
            ctx.hasDestReqErr = true;
            resolve();
        });
        req.on('fatalError', err => {
            error(ctx, err);
            resolve();
        });
        req.on('socketHangUp', () => {
            ctx.req.socket.end();
            resolve();
        });
    });
}
function error(ctx, err) {
    if (ctx.isPage && !ctx.isIframe) ctx.session.handlePageError(ctx, err);else if (ctx.isFetch || ctx.isXhr) ctx.req.destroy();else ctx.closeWithError(500, err.toString());
}
function callResponseEventCallbackForProcessedRequest(ctx, configureResponseEvent) {
    return __awaiter(this, void 0, void 0, function* () {
        const responseInfo = requestEventInfo.createResponseInfo(ctx);
        const preparedResponseInfo = requestEventInfo.prepareEventData(responseInfo, configureResponseEvent.opts);
        const responseEvent = new _responseEvent2.default(configureResponseEvent._requestFilterRule, preparedResponseInfo);
        yield ctx.session.callRequestEventCallback(_names2.default.onResponse, configureResponseEvent._requestFilterRule, responseEvent);
    });
}
function callOnRequestEventCallback(ctx, rule, reqInfo) {
    return __awaiter(this, void 0, void 0, function* () {
        const requestEvent = new _requestEvent2.default(ctx, rule, reqInfo);
        yield ctx.session.callRequestEventCallback(_names2.default.onRequest, rule, requestEvent);
    });
}
function callOnResponseEventCallbackForFailedSameOriginCheck(ctx, rule, configureOpts) {
    return __awaiter(this, void 0, void 0, function* () {
        const responseInfo = requestEventInfo.createResponseInfo(ctx);
        responseInfo.statusCode = _sameOriginCheckFailedStatusCode2.default;
        const preparedResponseInfo = requestEventInfo.prepareEventData(responseInfo, configureOpts);
        const responseEvent = new _responseEvent2.default(rule, preparedResponseInfo);
        yield ctx.session.callRequestEventCallback(_names2.default.onResponse, rule, responseEvent);
    });
}
function callOnConfigureResponseEventForNonProcessedRequest(ctx) {
    return __awaiter(this, void 0, void 0, function* () {
        yield ctx.forEachRequestFilterRule(rule => __awaiter(this, void 0, void 0, function* () {
            const configureResponseEvent = new _configureResponseEvent2.default(ctx, rule, _configureResponseEventOptions2.default.DEFAULT);
            yield ctx.session.callRequestEventCallback(_names2.default.onConfigureResponse, rule, configureResponseEvent);
            ctx.onResponseEventData.push({ rule, opts: configureResponseEvent.opts });
        }));
    });
}
function callOnResponseEventCallbackWithBodyForNonProcessedRequest(ctx, onResponseEventDataWithBody) {
    return __awaiter(this, void 0, void 0, function* () {
        const destResBodyCollectorStream = new _stream.PassThrough();
        ctx.destRes.pipe(destResBodyCollectorStream);
        (0, _promisifyStream2.default)(destResBodyCollectorStream).then(data => __awaiter(this, void 0, void 0, function* () {
            ctx.saveNonProcessedDestResBody(data);
            const responseInfo = requestEventInfo.createResponseInfo(ctx);
            yield Promise.all(onResponseEventDataWithBody.map(({ rule, opts }) => __awaiter(this, void 0, void 0, function* () {
                const preparedResponseInfo = requestEventInfo.prepareEventData(responseInfo, opts);
                const responseEvent = new _responseEvent2.default(rule, preparedResponseInfo);
                yield ctx.session.callRequestEventCallback(_names2.default.onResponse, rule, responseEvent);
            })));
            (0, _buffer.toReadableStream)(data).pipe(ctx.res);
        }));
    });
}
function callOnResponseEventCallbackWithoutBodyForNonProcessedResource(ctx, onResponseEventDataWithoutBody) {
    return __awaiter(this, void 0, void 0, function* () {
        const responseInfo = requestEventInfo.createResponseInfo(ctx);
        yield Promise.all(onResponseEventDataWithoutBody.map(item => __awaiter(this, void 0, void 0, function* () {
            const preparedResponseInfo = requestEventInfo.prepareEventData(responseInfo, item.opts);
            const responseEvent = new _responseEvent2.default(item.rule, preparedResponseInfo);
            yield ctx.session.callRequestEventCallback(_names2.default.onResponse, item.rule, responseEvent);
        })));
        ctx.destRes.pipe(ctx.res);
    });
}
function callOnResponseEventCallbackForMotModifiedResource(ctx) {
    return __awaiter(this, void 0, void 0, function* () {
        const responseInfo = requestEventInfo.createResponseInfo(ctx);
        yield Promise.all(ctx.onResponseEventData.map(item => __awaiter(this, void 0, void 0, function* () {
            const preparedResponseInfo = requestEventInfo.prepareEventData(responseInfo, item.opts);
            const responseEvent = new _responseEvent2.default(item.rule, preparedResponseInfo);
            yield ctx.session.callRequestEventCallback(_names2.default.onResponse, item.rule, responseEvent);
        })));
        ctx.res.end();
    });
}