'use strict';

exports.__esModule = true;

var _mustache = require('mustache');

var _mustache2 = _interopRequireDefault(_mustache);

var _readFileRelative = require('read-file-relative');

var _events = require('events');

var _url = require('url');

var _cookies = require('./cookies');

var _cookies2 = _interopRequireDefault(_cookies);

var _storage = require('../upload/storage');

var _storage2 = _interopRequireDefault(_storage);

var _command = require('./command');

var _command2 = _interopRequireDefault(_command);

var _generateUniqueId = require('../utils/generate-unique-id');

var _generateUniqueId2 = _interopRequireDefault(_generateUniqueId);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var __awaiter = undefined && undefined.__awaiter || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) {
            try {
                step(generator.next(value));
            } catch (e) {
                reject(e);
            }
        }
        function rejected(value) {
            try {
                step(generator["throw"](value));
            } catch (e) {
                reject(e);
            }
        }
        function step(result) {
            result.done ? resolve(result.value) : new P(function (resolve) {
                resolve(result.value);
            }).then(fulfilled, rejected);
        }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};

const TASK_TEMPLATE = (0, _readFileRelative.readSync)('../client/task.js.mustache');
class Session extends _events.EventEmitter {
    constructor(uploadsRoot) {
        super();
        this.uploadStorage = new _storage2.default(uploadsRoot);
        this.id = (0, _generateUniqueId2.default)();
        this.cookies = new _cookies2.default();
        this.proxy = null;
        this.externalProxySettings = null;
        this.pageLoadCount = 0;
        this.requireStateSwitch = false;
        this.pendingStateSnapshot = null;
        this.injectable = {
            scripts: ['/hammerhead.js'],
            styles: []
        };
        this.requestEventListeners = new Map();
        this.mocks = new Map();
    }
    // State
    getStateSnapshot() {
        return {
            cookies: this.cookies.serializeJar(),
            storages: null
        };
    }
    useStateSnapshot(snapshot) {
        // NOTE: we don't perform state switch immediately, since there might be
        // pending requests from current page. Therefore, we perform switch in
        // onPageRequest handler when new page is requested.
        this.requireStateSwitch = true;
        this.pendingStateSnapshot = snapshot || {
            cookies: null,
            storages: {
                localStorage: '[[],[]]',
                sessionStorage: '[[],[]]'
            }
        };
    }
    handleServiceMessage(msg, serverInfo) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this[msg.cmd]) return yield this[msg.cmd](msg, serverInfo);
            throw new Error('Malformed service message or message handler is not implemented');
        });
    }
    _fillTaskScriptTemplate({ serverInfo, isFirstPageLoad, referer, cookie, iframeTaskScriptTemplate, payloadScript }) {
        referer = referer || '{{{referer}}}';
        cookie = cookie || '{{{cookie}}}';
        iframeTaskScriptTemplate = iframeTaskScriptTemplate || '{{{iframeTaskScriptTemplate}}}';
        const domain = serverInfo.domain,
              crossDomainPort = serverInfo.crossDomainPort;

        return _mustache2.default.render(TASK_TEMPLATE, {
            sessionId: this.id,
            serviceMsgUrl: domain + '/messaging',
            forceProxySrcForImage: this.hasRequestEventListeners(),
            crossDomainPort,
            isFirstPageLoad,
            referer,
            cookie,
            iframeTaskScriptTemplate,
            payloadScript
        });
    }
    getIframeTaskScriptTemplate(serverInfo) {
        const taskScriptTemplate = this._fillTaskScriptTemplate({
            serverInfo,
            isFirstPageLoad: false,
            referer: null,
            cookie: null,
            iframeTaskScriptTemplate: null,
            payloadScript: this._getIframePayloadScript(true)
        });
        return JSON.stringify(taskScriptTemplate);
    }
    getTaskScript({ referer, cookieUrl, serverInfo, isIframe, withPayload }) {
        const cookies = JSON.stringify(this.cookies.getClientString(cookieUrl));
        let payloadScript = '';
        if (withPayload) payloadScript = isIframe ? this._getIframePayloadScript(false) : this._getPayloadScript();
        const taskScript = this._fillTaskScriptTemplate({
            serverInfo,
            isFirstPageLoad: this.pageLoadCount === 0,
            referer,
            cookie: cookies,
            iframeTaskScriptTemplate: this.getIframeTaskScriptTemplate(serverInfo),
            payloadScript
        });
        this.pageLoadCount++;
        return taskScript;
    }
    setExternalProxySettings(proxySettings) {
        if (typeof proxySettings === 'string') proxySettings = { url: proxySettings };
        proxySettings = proxySettings || {};
        var _proxySettings = proxySettings;
        const url = _proxySettings.url,
              bypassRules = _proxySettings.bypassRules;

        const parsedUrl = typeof url === 'string' ? (0, _url.parse)('http://' + url) : null;
        let settings = null;
        if (parsedUrl && parsedUrl.host) {
            settings = {
                host: parsedUrl.host,
                hostname: parsedUrl.hostname
            };
            if (bypassRules) settings.bypassRules = bypassRules;
            if (parsedUrl.port) settings.port = parsedUrl.port;
            if (parsedUrl.auth) {
                settings.proxyAuth = parsedUrl.auth;
                settings.authHeader = 'Basic ' + Buffer.from(parsedUrl.auth).toString('base64');
            }
        }
        this.externalProxySettings = settings;
    }
    onPageRequest(ctx) {
        if (this.requireStateSwitch) {
            this.cookies.setJar(this.pendingStateSnapshot.cookies);
            ctx.restoringStorages = this.pendingStateSnapshot.storages;
            this.requireStateSwitch = false;
            this.pendingStateSnapshot = null;
        }
    }
    // Request hooks
    hasRequestEventListeners() {
        return !!this.requestEventListeners.size;
    }
    addRequestEventListeners(requestFilterRule, eventListeners) {
        this.requestEventListeners.set(requestFilterRule, eventListeners);
    }
    removeRequestEventListeners(requestFilterRule) {
        this.requestEventListeners.delete(requestFilterRule);
    }
    getRequestFilterRules(requestInfo) {
        const rulesArray = Array.from(this.requestEventListeners.keys());
        return rulesArray.filter(rule => rule.match(requestInfo));
    }
    callRequestEventCallback(eventName, requestFilterRule, eventData) {
        return __awaiter(this, void 0, void 0, function* () {
            const eventListeners = this.requestEventListeners.get(requestFilterRule);
            const targetRequestEventCallback = eventListeners[eventName];
            if (typeof targetRequestEventCallback === 'function') yield targetRequestEventCallback(eventData);
        });
    }
    setMock(requestFilterRule, mock) {
        this.mocks.set(requestFilterRule, mock);
    }
    getMock(requestFilterRule) {
        return this.mocks.get(requestFilterRule);
    }
    // Service message handlers
    [_command2.default.uploadFiles](msg) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.uploadStorage.store(msg.fileNames, msg.data);
        });
    }
    [_command2.default.getUploadedFiles](msg) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.uploadStorage.get(msg.filePaths);
        });
    }
}
exports.default = Session;
module.exports = exports['default'];